 /*    
  * Copyright (c) 2023, SHANGHAI FUDAN MICROELECTRONICS GROUP CO., LTD.(FUDAN MICROELECTRONICS./ FUDAN MICRO.)    
  * All rights reserved.    
  *    
  * Processor:                   FM33FG0xxA    
  * http:                        http://www.fmdevelopers.com.cn/    
  *    
  * Redistribution and use in source and binary forms, with or without    
  * modification, are permitted provided that the following conditions are met    
  *    
  * 1. Redistributions of source code must retain the above copyright notice,    
  *    this list of conditions and the following disclaimer.    
  *    
  * 2. Redistributions in binary form must reproduce the above copyright notice,    
  *    this list of conditions and the following disclaimer in the documentation    
  *    and/or other materials provided with the distribution.    
  *    
  * 3. Neither the name of the copyright holder nor the names of its contributors    
  *    may be used to endorse or promote products derived from this software    
  *    without specific prior written permission.    
  *    
  * 4. To provide the most up-to-date information, the revision of our documents     
  *    on the World Wide Web will be the most Current. Your printed copy may be      
  *    an earlier revision. To verify you have the latest information avaliable,    
  *    refer to: http://www.fmdevelopers.com.cn/.    
  *    
  * THIS SOFTWARE IS PROVIDED BY FUDAN MICRO "AS IS" AND ANY EXPRESSED     
  * ORIMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES     
  * OF MERCHANTABILITY NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE    
  * ARE DISCLAIMED.IN NO EVENT SHALL FUDAN MICRO OR ITS CONTRIBUTORS BE LIABLE     
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL     
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS     
  * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER    
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,     
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISINGIN ANY WAY OUT OF THE     
  * USE OF THIS SOFTWARE, EVEN IF ADVISED OFTHE POSSIBILITY OF SUCH DAMAGE.    
 */    
/* Includes ------------------------------------------------------------------*/
#include "mf_config.h"
#include "fm33fg0xxa_fl.h"

/* Private function prototypes -----------------------------------------------*/

/**
  * @brief  The application entry point.
  * @retval int
  */
void MF_Clock_Init(void)
{
    /* MCU Configuration--------------------------------------------------------*/

    /* Reset of all peripherals, Initializes the Flash interface and the Systick. */

    /* System interrupt init*/

    /* Initialize all configured peripherals */
}

/**
  * @brief System Clock Configuration
  * @retval None
  */
void MF_SystemClock_Config(void)
{
    
}

void MF_LPTIM16_Init(void)
{
    FL_LPTIM16_InitTypeDef lptim16;
    FL_NVIC_ConfigTypeDef nvic;

    FL_LPTIM16_StructInit(&lptim16);

    lptim16.mode = FL_LPTIM16_OPERATION_MODE_NORMAL;
    lptim16.clockSource = FL_CMU_LPTIM16_CLK_SOURCE_APBCLK;
    lptim16.prescalerClockSource = FL_LPTIM16_CLK_SOURCE_INTERNAL;
    lptim16.prescaler = FL_LPTIM16_PSC_DIV8;
    lptim16.autoReload = 65536 - 1;
    lptim16.onePulseMode = FL_LPTIM16_ONE_PULSE_MODE_CONTINUOUS;
    (void) FL_LPTIM16_Init(LPTIM16, &lptim16);

    nvic.preemptPriority = 2;
    FL_NVIC_Init(&nvic, LPTIM_BSTIM_IRQn);

    FL_LPTIM16_ClearFlag_Update(LPTIM16);
    FL_LPTIM16_EnableIT_Update(LPTIM16);
}


void MF_PMU_Init(void)
{
    FL_PMU_SleepInitTypeDef    defaultInitStruct;
    /* 当使用到SVD作为唤醒源时，PMU_CR只允许配置Normal Sleep模式 */
    defaultInitStruct.deepSleep       = FL_PMU_SLEEP_MODE_NORMAL;  /* Normal Sleep */ 
    defaultInitStruct.wakeupFrequency = FL_PMU_RCHF_WAKEUP_FREQ_8MHZ;
    defaultInitStruct.wakeupDelay     = FL_PMU_WAKEUP_DELAY_2US;

    (void)FL_PMU_Sleep_Init(PMU, &defaultInitStruct);
}

/**
  * @brief  配置CAN控制器
  * @param  void
  * @retval void
  */
void MF_Config_Init(void)
{
    /* 配置LITIM16 */
    MF_LPTIM16_Init();
    /* 配置唤醒后的工作环境  */
    MF_PMU_Init();
}



/**
  * @brief  This function is executed in case of error occurrence.
  * @retval None
  */
void Error_Handler(void)
{
    /* USER CODE BEGIN Error_Handler_Debug */
    /* User can add his own implementation to report the HAL error return state */

    /* USER CODE END Error_Handler_Debug */
}

#ifdef  USE_FULL_ASSERT
/**
  * @brief  Reports the name of the source file and the source line number
  *         where the assert_param error has occurred.
  * @param  file: pointer to the source file name
  * @param  line: assert_param error line source number
  * @retval None
  */
void assert_failed(uint8_t *file, uint32_t line)
{ 
  /* USER CODE BEGIN Assert_Failed */
    /* User can add his own implementation to report the file name and line number,
       tex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */
  /* USER CODE END Assert_Failed */
}
#endif /* USE_FULL_ASSERT */

/************************ (C) COPYRIGHT FMSH *****END OF FILE****/
